﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using TilesArrangeDemo.Resources;
using System.Windows.Media;
using Microsoft.Expression.Interactivity.Layout;
using System.Windows.Threading;
using System.Diagnostics;

namespace TilesArrangeDemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        // 构造函数
        public MainPage()
        {
            InitializeComponent();
            this.Loaded += MainPage_Loaded;
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            InitDic();
        }

        // Size : 资源素距离父元素左上角的距离
        Dictionary<Point, Border> dic = new Dictionary<Point, Border>();
        int childWidth;

        // 初始化一个字典集合，Key：border 元素的中心点坐标，value：border 自己
        void InitDic()
        {
            dic.Clear();

            foreach (var item in ContentPanel.Children)
            {
                childWidth = (int)item.DesiredSize.Width;

                Border border = item as Border;
                int row = Grid.GetRow(border);
                int col = Grid.GetColumn(border);
                Point position = new Point(col * childWidth + childWidth / 2, row * childWidth + childWidth / 2);
                border.Tag = position;

                dic.Add(position, border);

                Debug.WriteLine("point-  x:" + (col * childWidth + childWidth / 2) + "  y:" + (row * childWidth + childWidth / 2));
            }
        }

        Border borderTemp;
        CompositeTransform transformTemp;
        private void Border_Hold(object sender, System.Windows.Input.GestureEventArgs e)
        {
            e.Handled = true;

            borderTemp = sender as Border;

            // 注册 tile 右上角“图钉”图片的单击事件
            (borderTemp.Child as TileControl).Assets_thumb_png.Tap += Assets_thumb_png_Tap;

            transformTemp = new CompositeTransform();

            transformTemp.TranslateX = x;
            transformTemp.TranslateY = y;

            borderTemp.RenderTransform = transformTemp;

            // 拖动 border 时，改变它的坐标
            borderTemp.ManipulationDelta += borderTemp_ManipulationDelta;

            // 当手指离开 border 时，判断各个 tile 中心点的距离
            borderTemp.MouseLeftButtonUp += borderTemp_MouseLeftButtonUp;

            if (borderTemp.Child != null)
            {
                foreach (var item in ContentPanel.Children)
                {
                    Border border = item as Border;
                    if (border.Child != null)
                    {
                        TileControl tile = border.Child as TileControl;
                        // 被选中的 tile 进入 CheckedVisualState 视图状态，其余的进入 FloatVisualState 视图状态
                        if (tile == (borderTemp.Child as TileControl))
                        {
                            tile.Checked();                            
                            border.IsHitTestVisible = true;
                        }
                        else
                        {
                            tile.Float();
                            // 当处于漂浮状态时，不再接收“单击”等屏幕事件
                            border.IsHitTestVisible = false;
                        }
                    }
                }
            }
        }

        // 单击“图钉”时，所有 tile 恢复 NormalVisualState 状态
        void Assets_thumb_png_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            e.Handled = true;
            foreach (var item in ContentPanel.Children)
            {
                Border border = item as Border;

                border.IsHitTestVisible = true;
                (border.Child as TileControl).Reset();
                border.ManipulationDelta -= borderTemp_ManipulationDelta;
            }
            x = 0;
            y = 0;
        }

        int x;
        int y;        
        void borderTemp_MouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            e.Handled = true;

            DateTime timeStart = DateTime.Now;

            Debug.WriteLine("Translation.X: " + transformTemp.TranslateX);
            Debug.WriteLine("Translation.Y: " + transformTemp.TranslateY);

            Point pointSource = (Point)borderTemp.Tag;

            Point point = new Point(transformTemp.TranslateX + pointSource.X, transformTemp.TranslateY + pointSource.Y);

            bool IsSwaped = false;

            // 当手指离开屏幕时，判断选中的 tile 的中心点和其它中心点的直线距离，
            // 如果距离小于 40px，则两个 tile 互换
            foreach (Point position in dic.Keys)
            {
                Border border2 = dic[position];
                if (borderTemp != border2)
                {
                    x = (int)(point.X - position.X);
                    y = (int)(point.Y - position.Y);

                    int distance = (int)Math.Sqrt(x * x + y * y);

                    if (distance < 40)
                    {
                        // 交换两个 tile 的位置
                        SwapBorder(borderTemp, border2);
                        IsSwaped = true;
                        break;
                    }
                }
            }

            Debug.WriteLine(DateTime.Now - timeStart);

            if (!IsSwaped)
            {
                transformTemp.TranslateX = 0;
                transformTemp.TranslateY = 0;
            }
        }

        // 手指移动时，同时移动选中 tile 的位置
        void borderTemp_ManipulationDelta(object sender, System.Windows.Input.ManipulationDeltaEventArgs e)
        {
            e.Handled = true;
            transformTemp.TranslateX += e.DeltaManipulation.Translation.X;
            transformTemp.TranslateY += e.DeltaManipulation.Translation.Y;

            Debug.WriteLine(" e.DeltaManipulation.Translation.X :" + e.DeltaManipulation.Translation.X);
            Debug.WriteLine(" e.DeltaManipulation.Translation.Y :" + e.DeltaManipulation.Translation.Y);
        }

        // 交换 border
        void SwapBorder(Border border1, Border border2)
        {
            int colNum1 = Grid.GetColumn(border1);
            int rowNum1 = Grid.GetRow(border1);

            int colNum2 = Grid.GetColumn(border2);
            int rowNum2 = Grid.GetRow(border2);

            Grid.SetColumn(border2, colNum1);
            Grid.SetRow(border2, rowNum1);

            border1.Hold -= Border_Hold;
            ContentPanel.Children.Remove(border1);

            Border borderNew = BorderClone(border1);

            Grid.SetColumn(borderNew, colNum2);
            Grid.SetRow(borderNew, rowNum2);
            ContentPanel.Children.Add(borderNew);

            Border_Hold(borderNew, new System.Windows.Input.GestureEventArgs());

            borderNew.UpdateLayout();
            InitDic();
        }

        // 克隆选中 border 的副本
        Border BorderClone(Border srcBorder)
        {
            TileControl srcTile = srcBorder.Child as TileControl;

            Border borderNew = new Border();
            borderNew.Hold += Border_Hold;

            TileControl tile = new TileControl { TileBackground = srcTile.TileBackground };

            borderNew.Child = tile;

            return borderNew;
        }
    }
}